// (C) Copyright 2002-2012 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

//-----------------------------------------------------------------------------
//----- acrxEntryPoint.cpp
//-----------------------------------------------------------------------------
#include "StdAfx.h"
#include "resource.h"

//-----------------------------------------------------------------------------
#define szRDS _RXST("")

//-----------------------------------------------------------------------------
//----- ObjectARX EntryPoint
class CCurvSurfInterApp : public AcRxArxApp {

public:
  CCurvSurfInterApp () : AcRxArxApp () {}

  virtual AcRx::AppRetCode On_kInitAppMsg (void *pkt) {
    // TODO: Load dependencies here

    // You *must* call On_kInitAppMsg here
    AcRx::AppRetCode retCode =AcRxArxApp::On_kInitAppMsg (pkt) ;
    acdbModelerStart();
    // TODO: Add your initialization code here

    return (retCode) ;
  }

  virtual AcRx::AppRetCode On_kUnloadAppMsg (void *pkt) {
    // TODO: Add your code here

    // You *must* call On_kUnloadAppMsg here
    AcRx::AppRetCode retCode =AcRxArxApp::On_kUnloadAppMsg (pkt) ;

    // TODO: Unload dependencies here

    return (retCode) ;
  }

  virtual void RegisterServerComponents () {

  }

  static void RivilisCheckInt () {
    Acad::ErrorStatus es;
    ads_name en; ads_point p;
    double level1 = -10.0;
    double level2 = +10.0;
    double step = 1.0;
    if (acedEntSel(_T("\n  : "), en, p) == RTNORM) {
      TCHAR buf1[32]; acdbRToS(level1,-1,-1, buf1, sizeof(buf1)-1);
      TCHAR buf2[32]; acdbRToS(level2,-1,-1, buf2, sizeof(buf2)-1);
      TCHAR buf3[32]; acdbRToS(step,-1,-1, buf3, sizeof(buf3)-1);
      AcString s1; s1.format(_T("\n         <%s>: "), buf1);
      AcString s2; s2.format(_T("\n         <%s>: "), buf2);
      if (acedGetReal(s1,&level1) != RTCAN && acedGetReal(s2,&level2) != RTCAN) {
        AcString s3; s3.format(_T("\n   <%s>: "), buf3);
        acedInitGet(RSG_NOZERO | RSG_NONEG, NULL);
        if (acedGetReal(s3,&step) == RTCAN) return;
        AcDbObjectId eId; acdbGetObjectId(eId, en);
        AcDbObjectPointer<AcDbCurve> pCurve(eId, AcDb::kForRead);
        if (pCurve.openStatus() == Acad::eWrongObjectType) {
          acutPrintf(_T("\n  .  , , , , ,  .."));
          return;
        }
        if (pCurve.openStatus() != Acad::eOk) {
          acutPrintf(_T("\n : %s"), acadErrorStatusText(pCurve.openStatus()));
          return;
        }
        // GE-,   AcDb-
        AcGeCurve3d *pGeCurve = NULL;
        if ((es = acdbConvertAcDbCurveToGelibCurve(pCurve, pGeCurve)) != Acad::eOk) {
          acutPrintf(_T("\nacdbConvertAcDbCurveToGelibCurve: %s"), acadErrorStatusText(es));
          return;
        }
        for (double z = min(level1, level2); z <= max(level1,level2); z += step) 
        {
          //  
          AcGePlane plan(AcGePoint3d(0,0,z), AcGeVector3d::kZAxis);
          AcGeIntersectError err;
          AcGeCurveSurfInt csint(*pGeCurve, plan);
          int n = csint.numIntPoints(err);
          if ((n = csint.numIntPoints(err)) != 0)
          {
            //   
            acdbCurDwg()->setPdmode(34);
            acutPrintf(_T("\n %d    %g"), n, z);
            AcDbBlockTableRecordPointer pBTR(acdbCurDwg()->currentSpaceId(), AcDb::kForWrite);
            if (pBTR.openStatus() == Acad::eOk) {
              for (int i = 0; i < n; i++) {
                AcDbPoint *pPoint = new AcDbPoint(csint.intPoint(i,err));
                pPoint->setDatabaseDefaults();
                pPoint->setColorIndex(2);
                pBTR->appendAcDbEntity(pPoint);
                pPoint->close();
              }
            }
          }
        }
        delete pGeCurve;
      } 
    } 
  }

} ;

//-----------------------------------------------------------------------------
IMPLEMENT_ARX_ENTRYPOINT(CCurvSurfInterApp)

ACED_ARXCOMMAND_ENTRY_AUTO(CCurvSurfInterApp, Rivilis, CheckInt, CheckInt, ACRX_CMD_MODAL, NULL)
